<?php
# Private[...]Provider -- password auth that doesn't reveal user names
#
# This class extends the MediaWiki-provided local password authentication
# provider to eliminate the distinction between "invalid user" and
# "valid user with wrong password".
#
# One reason for this is so we don't give a free clue to attackers when
# they have found a real user name. Another reason is so the
# AuthManagerLoginAuthenticateAudit hook gets called for both failure
# cases. (This is vital if we are using the hook function to log failures
# to drive fail2ban or similar.)
class PrivateWikiLocalPasswordPrimaryAuthenticationProvider extends
 MediaWiki\Auth\LocalPasswordPrimaryAuthenticationProvider {

   public function beginPrimaryAuthentication( array $reqs ) {
      # Start by calling the function from the parent class that
      # we're overriding.
      $rsp = parent::beginPrimaryAuthentication($reqs);

      # Fold all ABSTAIN and FAIL cases to a FAIL with a single message:
      if($rsp->status == MediaWiki\Auth\AuthenticationResponse::ABSTAIN ||
       $rsp->status == MediaWiki\Auth\AuthenticationResponse::FAIL) {
         return MediaWiki\Auth\AuthenticationResponse::newFail(
          wfMessage('authmanager-authn-no-primary'));
      }

      # Return the (possibly modified) response.
      return $rsp;
   }
}
